@@ -1,6 +1,6 @@
 """Constants for commonmeta-py"""
 
-from typing import Optional, TypedDict, List
+from typing import List, Optional, TypedDict
 
 
 class Commonmeta(TypedDict):
@@ -190,6 +190,7 @@ CM_TO_CR_TRANSLATIONS = {
     "JournalIssue": "JournalIssue",
     "JournalVolume": "JournalVolume",
     "Journal": "Journal",
+    "PeerReview": "PeerReview",
     "ProceedingsArticle": "ProceedingsArticle",
     "ProceedingsSeries": "ProceedingsSeries",
     "Proceedings": "Proceedings",
@@ -464,6 +465,30 @@ CM_TO_SO_TRANSLATIONS = {
     "Presentation": "PresentationDigitalDocument",
 }
 
+# source: https://api.openalex.org/works?group_by=type
+OA_TO_CM_TRANSLATIONS = {
+    "article": "Article",
+    "book": "Book",
+    "book-chapter": "BookChapter",
+    "dataset": "Dataset",
+    "dissertation": "Dissertation",
+    "editorial": "Document",
+    "erratum": "Other",
+    "grant": "Grant",
+    "letter": "Article",
+    "libguides": "InteractiveResource",
+    "other": "Other",
+    "paratext": "Component",
+    "peer-review": "PeerReview",
+    "preprint": "Article",
+    "reference-entry": "Other",
+    "report": "Report",
+    "retraction": "Other",
+    "review": "Article",
+    "standard": "Standard",
+    "supplementary-materials": "Component",
+}
+
 SO_TO_DC_RELATION_TYPES = {
     "citation": "References",
     "isBasedOn": "IsSupplementedBy",
@@ -527,6 +552,16 @@ DC_TO_CM_CONTAINER_TRANSLATIONS = {
     "Series": "Series",
 }
 
+OA_TO_CM_CONTAINER_TRANLATIONS = {
+    "journal": "Journal",
+    "repository": "Repository",
+    "conference": "Proceedings",
+    "ebook platform": "Book",
+    "book series": "BookSeries",
+    "metadata": "DataRepository",
+    "Other": "Repository",
+}
+
 DATACITE_CONTRIBUTOR_TYPES = {
     "ContactPerson": "ContactPerson",
     "DataCollector": "DataCollector",
@@ -664,3 +699,22 @@ ROR_TO_CROSSREF_FUNDER_ID_TRANSLATIONS = {
     "https://ror.org/00yjd3n13": "https://doi.org/10.13039/501100001711",
     "https://ror.org/04wfr2810": "https://doi.org/10.13039/501100003043",
 }
+
+COMMUNITY_TRANSLATIONS = {
+    "ai": "artificialintelligence",
+    "llms": "artificialintelligence",
+    "book%20review": "bookreview",
+    "bjps%20review%20of%20books": "bookreview",
+    "books": "bookreview",
+    "nachrichten": "news",
+    "opencitations": "researchassessment",
+    "papers": "researchblogging",
+    "urheberrecht": "copyright",
+    "workshop": "events",
+    "veranstaltungen": "events",
+    "veranstaltungshinweise": "events",
+    "asapbio": "preprints",
+    "biorxiv": "preprints",
+    "runiverse": "r",
+    "bericht": "report",
+}
