@@ -1,31 +1,46 @@
 """schema_org reader for commonmeta-py"""
-from typing import Optional
-import json
-from collections import defaultdict
-import requests
-from pydash import py_
-from bs4 import BeautifulSoup
 
-from ..utils import (
-    dict_to_spdx,
-    normalize_cc_url,
-    from_schema_org,
-    from_schema_org_creators,
-    normalize_id,
-    normalize_ids,
-    normalize_url,
-    name_to_fos,
-)
-from ..base_utils import wrap, compact, presence, parse_attributes, sanitize
+import io
+from collections import defaultdict
+from datetime import datetime
+from typing import Optional
+
+import orjson as json
+import pikepdf
+import requests
+from bs4 import BeautifulSoup
+from pydash import py_
+from requests.exceptions import ConnectionError
+
 from ..author_utils import get_authors
-from ..date_utils import get_iso8601_date, strip_milliseconds
-from ..doi_utils import doi_from_url, get_doi_ra
+from ..base_utils import compact, parse_attributes, presence, sanitize, wrap
 from ..constants import (
+    OG_TO_SO_TRANSLATIONS,
     SO_TO_CM_TRANSLATIONS,
     SO_TO_DC_RELATION_TYPES,
     SO_TO_DC_REVERSE_RELATION_TYPES,
     Commonmeta,
 )
+from ..date_utils import (
+    get_datetime_from_pdf_time,
+    get_iso8601_date,
+    strip_milliseconds,
+)
+from ..doi_utils import doi_from_url, get_doi_ra, validate_doi
+from ..readers.crossref_reader import get_crossref
+from ..readers.datacite_reader import get_datacite
+from ..translators import web_translator
+from ..utils import (
+    dict_to_spdx,
+    from_schema_org,
+    from_schema_org_creators,
+    get_language,
+    name_to_fos,
+    normalize_cc_url,
+    normalize_id,
+    normalize_ids,
+    normalize_url,
+)
 
 
 def get_schema_org(pid: str, **kwargs) -> dict:
@@ -33,58 +48,126 @@ def get_schema_org(pid: str, **kwargs) -> dict:
     if pid is None:
         return {"state": "not_found"}
     url = pid
-    response = requests.get(url, kwargs, timeout=10)
-    if response.status_code != 200:
-        return {"state": "not_found"}
+
+    # if pid represents a DOI, get metadata from Crossref or DataCite
+    if doi_from_url(pid):
+        return get_doi_meta(doi_from_url(pid))
+    try:
+        response = requests.get(url, timeout=10, allow_redirects=True, **kwargs)
+    except ConnectionError as error:
+        return {
+            "@id": url,
+            "@type": "WebPage",
+            "state": "not_found",
+            "via": "schema_org",
+            "errors": [str(error)],
+        }
+    if response.status_code >= 400:
+        if response.status_code in [404, 410]:
+            state = "not_found"
+        elif response.status_code in [401, 403]:
+            state = "forbidden"
+        else:
+            state = "bad_request"
+        return {"@id": url, "@type": "WebPage", "state": state, "via": "schema_org"}
+    elif response.headers.get("content-type") == "application/pdf":
+        try:
+            pdf = pikepdf.open(io.BytesIO(response.content))
+            with pdf.open_metadata() as meta:
+                if meta.get("/doi", None) is not None:
+                    return get_doi_meta(meta.get("/doi"))
+                date_modified = (
+                    get_datetime_from_pdf_time(meta.get("/ModDate"))
+                    if meta.get("/ModDate", None)
+                    else None
+                )
+                name = meta.get("/Title", None)
+                return compact(
+                    {
+                        "@id": url,
+                        "@type": "DigitalDocument",
+                        "via": "schema_org",
+                        "name": str(name),
+                        "datePublished": date_modified,
+                        "dateAccessed": datetime.now().isoformat("T", "seconds")
+                        if date_modified is None
+                        else None,
+                    }
+                )
+        except Exception as error:
+            print(error)
+            return {
+                "@id": url,
+                "@type": "WebPage",
+                "state": "bad_request",
+                "via": "schema_org",
+            }
 
     soup = BeautifulSoup(response.text, "html.parser")
-    # workaround for metadata not included with schema.org but in html meta tags
+
+    # load html meta tags
     data = get_html_meta(soup)
-    # load schema.org metadata
-    json_ld = soup.find("script", type="application/ld+json")
+    # load site-specific metadata
+    data |= web_translator(soup, url)
+
+    # load schema.org metadata. If there are multiple schema.org blocks, load them all,
+    # and pick the first one with a supported type
+    list = [
+        json.loads(x.text) for x in soup.find_all("script", type="application/ld+json")
+    ]
+    json_ld = next(
+        (i for i in list if i.get("@type", None) in SO_TO_CM_TRANSLATIONS),
+        None,
+    )
     if json_ld is not None:
-        data |= json.loads(json_ld.text)
+        data |= json_ld
 
-    # workaround if not all authors are included with schema.org (e.g. in Ghost metadata)
-    auth = soup.select("meta[name='citation_author']")
+    # if @id is a DOI, get metadata from Crossref or DataCite
+    if validate_doi(data.get("@id", None)):
+        return get_doi_meta(data.get("@id", None))
 
-    def format_author(author):
-        return {"name": author["content"]}
+    # if @id is None, use url
+    elif data.get("@id", None) is None:
+        data["@id"] = url
 
-    authors = [format_author(i) for i in auth]
+    # if @type is None, use WebSite
+    elif data.get("@type", None) is None:
+        data["@type"] = "WebSite"
 
+    # author and creator are synonyms
     if data.get("author", None) is None and data.get("creator", None) is not None:
         data["author"] = data["creator"]
-    if len(authors) > len(wrap(data.get("author", None))):
-        data["author"] = authors
-
-    return data
+    return data | {"via": "schema_org", "state": "findable"}
 
 
 def read_schema_org(data: Optional[dict], **kwargs) -> Commonmeta:
     """read_schema_org"""
-    if data is None:
-        return {"meta": None, "state": "not_found"}
+    if (
+        data is None
+        or isinstance(data, dict)
+        and data.get("state", None) in ["not_found", "forbidden", "bad_request"]
+    ):
+        return from_schema_org(data)
     meta = data
 
     read_options = kwargs or {}
 
-    id_ = meta.get("@id", None)
-    # if id.blank? && URI(meta.fetch('@id', '')).host == 'doi.org'
-    if id_ is None:
-        id_ = meta.get("identifier", None)
-    id_ = normalize_id(id_)
-    type_ = SO_TO_CM_TRANSLATIONS.get(meta.get("@type", None), 'Other')
+    _id = meta.get("@id", None)
+    if _id is None:
+        _id = meta.get("identifier", None)
+    _id = normalize_id(_id)
+    _type = SO_TO_CM_TRANSLATIONS.get(meta.get("@type", None), "WebPage")
     additional_type = meta.get("additionalType", None)
-    authors = meta.get("author", None) or meta.get("creator", None)
-    # Authors should be an object, if it's just a plain string don't try and parse it.
-    if not isinstance(authors, str):
-        creators = get_authors(from_schema_org_creators(wrap(authors)))
-    else:
-        creators = authors
-    contributors = presence(
+    url = normalize_url(meta.get("url", None)) or _id
+
+    # Authors should be list of objects or strings
+    authors = wrap(meta.get("author", None))
+    contributors = get_authors(from_schema_org_creators(authors))
+    contrib = presence(
         get_authors(from_schema_org_creators(wrap(meta.get("editor", None))))
     )
+    if contrib:
+        contributors = contributors + contrib
 
     if meta.get("name", None) is not None:
         titles = [{"title": meta.get("name")}]
@@ -94,52 +177,64 @@ def read_schema_org(data: Optional[dict], **kwargs) -> Commonmeta:
         titles = None
 
     date: dict = defaultdict(list)
-    date['published'] = strip_milliseconds(meta.get("datePublished", None))
-    date['updated'] = strip_milliseconds(meta.get("dateModified", None))
+    date["published"] = strip_milliseconds(meta.get("datePublished", None))
+    date["updated"] = strip_milliseconds(meta.get("dateModified", None))
+    # if no date is found, use today's date
+    if date == {"published": None, "updated": None}:
+        date["accessed"] = read_options.get(
+            "dateAccessed", None
+        ) or datetime.now().isoformat("T", "seconds")
 
     publisher = meta.get("publisher", None)
     if publisher is not None:
-        publisher = py_.omit(publisher, ["@type", "logo", "url", "disambiguatingDescription"])
+        publisher = py_.omit(
+            publisher, ["@type", "logo", "url", "disambiguatingDescription"]
+        )
 
     license_ = meta.get("license", None)
     if license_ is not None:
         license_ = normalize_cc_url(license_)
         license_ = dict_to_spdx({"url": license_}) if license_ else None
 
-    if type_ == "Dataset":
-        url = parse_attributes(
-            from_schema_org(meta.get('includedInDataCatalog', None)), content="url", first=True
+    if _type == "Dataset":
+        container_url = parse_attributes(
+            from_schema_org(meta.get("includedInDataCatalog", None)),
+            content="url",
+            first=True,
         )
         container = compact(
             {
                 "type": "DataRepository",
                 "title": parse_attributes(
-                    from_schema_org(meta.get('includedInDataCatalog', None)), content="name", first=True
+                    from_schema_org(meta.get("includedInDataCatalog", None)),
+                    content="name",
+                    first=True,
                 ),
-                "identifier": url,
-                "identifierType": "URL" if url is not None else None,
+                "identifier": container_url,
+                "identifierType": "URL" if container_url is not None else None,
                 "volume": meta.get("volumeNumber", None),
                 "issue": meta.get("issueNumber", None),
                 "firstPage": meta.get("pageStart", None),
                 "lastPage": meta.get("pageEnd", None),
             }
         )
-    elif type_ == "Article":
+    elif _type in ["Article", "BlogPost"]:
+        container_type = "Blog" if _type == "BlogPost" else "Periodical"
         issn = py_.get(meta, "isPartOf.issn")
-        url = py_.get(meta, "publisher.url")
+        container_url = py_.get(meta, "publisher.url")
         container = compact(
             {
-                "type": "Blog",
+                "type": container_type,
                 "title": py_.get(meta, "isPartOf.name"),
                 "identifier": issn
                 if issn is not None
-                else url
-                if url is not None
+                else container_url
+                if container_url is not None
                 else None,
                 "identifierType": "ISSN"
                 if issn is not None
                 else "URL"
-                if url is not None
+                if container_url is not None
                 else None,
             }
         )
@@ -154,7 +249,7 @@ def read_schema_org(data: Optional[dict], **kwargs) -> Commonmeta:
     descriptions = [
         {
             "description": sanitize(i),
-            "descriptionType": "Abstract",
+            "type": "Abstract",
         }
         for i in wrap(meta.get("description"))
     ]
@@ -179,27 +274,28 @@ def read_schema_org(data: Optional[dict], **kwargs) -> Commonmeta:
     geo_locations = [
         schema_org_geolocation(i) for i in wrap(meta.get("spatialCoverage", None))
     ]
-    alternate_identifiers = None
-    provider = get_doi_ra(id_) if doi_from_url(id_) else parse_attributes(
-            meta.get("provider", None), content="name", first=True
-        )
-    state = None
+    identifiers = None
+    provider = (
+        get_doi_ra(_id)
+        if doi_from_url(_id)
+        else parse_attributes(meta.get("provider", None), content="name", first=True)
+    )
+    state = "findable"
 
     return {
         # required attributes
-        "id": id_,
-        "type": type_,
-        "url": normalize_url(meta.get("url", None)),
-        "creators": creators,
+        "id": _id,
+        "type": _type,
+        "url": url,
+        "contributors": presence(contributors),
         "titles": titles,
         "publisher": publisher,
         "date": compact(date),
         # recommended and optional attributes
         "additional_type": additional_type,
         "subjects": presence(subjects),
-        "contributors": contributors,
-        "language": language,
-        "alternate_identifiers": alternate_identifiers,
+        "language": get_language(language),
+        "identifiers": identifiers,
         "sizes": None,
         "formats": None,
         "version": meta.get("version", None),
@@ -215,6 +311,16 @@ def read_schema_org(data: Optional[dict], **kwargs) -> Commonmeta:
     } | read_options
 
 
+def get_doi_meta(doi: str) -> Optional[dict]:
+    """get_doi_meta"""
+    ra = get_doi_ra(doi)
+    if ra == "Crossref":
+        return get_crossref(doi)
+    elif ra == "DataCite":
+        return get_datacite(doi)
+    return None
+
+
 def schema_org_related_item(meta, relation_type=None):
     """Related items"""
     normalize_ids(
@@ -281,11 +387,11 @@ def schema_org_geolocation(geo_location: Optional[dict]) -> Optional[dict]:
     if not isinstance(geo_location, dict):
         return None
 
-    type_ = py_.get(geo_location, "geo.@type")
+    _type = py_.get(geo_location, "geo.@type")
     longitude = py_.get(geo_location, "geo.longitude")
     latitude = py_.get(geo_location, "geo.latitude")
 
-    if type_ == "GeoCoordinates":
+    if _type == "GeoCoordinates":
         return {
             "geoLocationPoint": {"pointLongitude": longitude, "pointLatitude": latitude}
         }
@@ -298,26 +404,47 @@ def get_html_meta(soup):
     pid = (
         soup.select_one("meta[name='citation_doi']")
         or soup.select_one("meta[name='dc.identifier']")
+        or soup.select_one("meta[name='DC.identifier']")
+        or soup.select_one("meta[name='bepress_citation_doi']")
         or soup.select_one('[rel="canonical"]')
     )
     if pid is not None:
-        data["@id"] = pid.get("content", None) or pid.get("href", None)
+        pid = pid.get("content", None) or pid.get("href", None)
+        data["@id"] = normalize_id(pid)
 
-    type_ = soup.select_one("meta[property='og:type']")
-    data["@type"] = type_["content"].capitalize() if type_ else None
+    _type = soup.select_one("meta[name='dc.type']") or soup.select_one(
+        "meta[name='DC.type']"
+    )
+    data["@type"] = _type["content"].capitalize() if _type else None
+    if _type is None:
+        _type = soup.select_one("meta[property='og:type']")
+        data["@type"] = OG_TO_SO_TRANSLATIONS.get(_type["content"]) if _type else None
 
-    url = soup.select_one("meta[property='og:url']")
+    url = soup.select_one("meta[property='og:url']") or soup.select_one(
+        "meta[name='twitter:url']"
+    )
     data["url"] = url["content"] if url else None
+    if pid is None and url is not None:
+        data["@id"] = url["content"]
 
     title = (
         soup.select_one("meta[name='citation_title']")
         or soup.select_one("meta[name='dc.title']")
+        or soup.select_one("meta[name='DC.title']")
         or soup.select_one("meta[property='og:title']")
+        or soup.select_one("meta[name='twitter:title']")
+        or soup.select_one("meta[name='title']")
     )
     data["name"] = title["content"] if title else None
 
+    author = soup.select("meta[name='citation_author']")
+    data["author"] = [i["content"] for i in author] if author else None
+
     description = soup.select_one("meta[name='citation_abstract']") or soup.select_one(
         "meta[name='dc.description']"
+        or soup.select_one("meta[property='og:description']")
+        or soup.select_one("meta[name='twitter:description']")
+        or soup.select_one("meta[name='description']")
     )
     data["description"] = description["content"] if description else None
 
@@ -326,13 +453,21 @@ def get_html_meta(soup):
         str(keywords["content"]).replace(";", ",").rstrip(", ") if keywords else None
     )
 
-    date_published = soup.select_one(
-        "meta[name='citation_publication_date']"
-    ) or soup.select_one("meta[name='dc.date']")
+    date_published = (
+        soup.select_one("meta[name='citation_publication_date']")
+        or soup.select_one("meta[name='dc.date']")
+        or soup.select_one("meta[property='article:published_time']")
+    )
     data["datePublished"] = (
         get_iso8601_date(date_published["content"]) if date_published else None
     )
-
+    date_modified = soup.select_one(
+        "meta[property='og:updated_time']"
+        or soup.select_one("meta[property='article:modified_time']")
+    )
+    data["dateModified"] = (
+        get_iso8601_date(date_modified["content"]) if date_modified else None
+    )
     license_ = soup.select_one("meta[name='dc.rights']")
     data["license"] = license_["content"] if license_ else None
 
@@ -342,9 +477,11 @@ def get_html_meta(soup):
     if lang is not None:
         data["inLanguage"] = lang["content"]
     else:
-        lang = soup.select_one("html")["lang"]
-        if lang is not None:
-            data["inLanguage"] = lang
+        html = soup.select_one("html")
+        if html is not None:
+            lang = html.get("lang", None)
+            if lang is not None:
+                data["inLanguage"] = lang
 
     publisher = soup.select_one("meta[property='og:site_name']")
     data["publisher"] = {"name": publisher["content"]} if publisher else None
